/** @file MembraneMultiLinear.h
 * @brief Contains the interface of the MembraneMulti22LinearCoflow and MembraneMulti22LinearCounterflow classes
 *
 * @attention All rights reserved; do not distribute without permission.
 * @author (C) Copyright 2011-2012 Paolo Greppi Universita di Genova; (C) Copyright 2012-2024 Paolo Greppi simevo s.r.l.
 */

#ifndef MEMBRANEMULTILINEAR_H
#define MEMBRANEMULTILINEAR_H

/* SYSTEM INCLUDES */
//

/* PROJECT INCLUDES */
#include <libpf/flowsheet/FlowSheet.h>

/* LOCAL INCLUDES */

/* FORWARD REFERENCES */
//

// /////////////////////////////////////////////////////////////////////////////
// //////////////////////// MembraneMulti22LinearCoflow ////////////////////////
// /////////////////////////////////////////////////////////////////////////////

/// Two-feeds, two-products, linear average, co-flow, multi-stage membrane model
/// Options:
///  - stages total number of stages, default value is 10
class MembraneMulti22LinearCoflow : virtual public Model, public FlowSheet {
public:
  // LIFECYCLE  
  MembraneMulti22LinearCoflow(Libpf::Persistency::Defaults defaults, uint32_t id=0, Persistency *persistency=nullptr, Persistent *parent=nullptr, Persistent *root=nullptr);
  
  // CUSTOM FLOWSHEET VARIABLES
  // inputs
  Quantity A; ///< membrane area
  Quantity deltaPretentate; ///< retentate compartment pressure drop
  Quantity deltaPpermeate; ///< permeate compartment pressure drop
  std::vector<Quantity> permeance; ///< permeance for each component

  // computed
  Quantity stageCut; ///< fraction of feed to permeate
  std::vector<Quantity> outSplit;  ///< component split fractions to permeate

  Integer stages; ///< total number of stages

  // OPERATIONS
  // mandatory
  void setup(void);
  const std::string &type(void) const { return type_; }
  void makeUserEquations(std::list<Assignment *>::iterator &p);
  void pre(Calculatable::SolutionMode solutionMode, int level);
  void post(Calculatable::SolutionMode solutionMode, int level);
  // non-mandatory
  // bool supportsSimultaneous(void) { return false; }
  // int maximumIterations(void) { return 1; }
private:
  static std::string type_;
  std::string stagename(int i);  
  const std::string reteout(void);
  const std::string permout(void);
};  // class MembraneMulti22LinearCoflow

// /////////////////////////////////////////////////////////////////////////////
// ////////////////////// MembraneMulti22LinearCounterflow /////////////////////
// /////////////////////////////////////////////////////////////////////////////

/// Two-feeds, two-products, linear average, counter-flow, multi-stage membrane model
/// Options:
///  - stages total number of stages, default value is 10
class MembraneMulti22LinearCounterflow : virtual public Model, public FlowSheet {
public:
  // LIFECYCLE  
  MembraneMulti22LinearCounterflow(Libpf::Persistency::Defaults defaults, uint32_t id=0, Persistency *persistency=nullptr, Persistent *parent=nullptr, Persistent *root=nullptr);
  
  // CUSTOM FLOWSHEET VARIABLES
  // inputs
  Quantity A; ///< membrane area
  Quantity deltaPretentate; ///< retentate compartment pressure drop
  Quantity deltaPpermeate; ///< permeate compartment pressure drop
  std::vector<Quantity> permeance; ///< permeance for each component

  // computed
  Quantity stageCut; ///< fraction of feed to permeate
  std::vector<Quantity> outSplit;  ///< component split fractions to permeate

  Integer stages; ///< total number of stages
  
  // OPERATIONS
  // mandatory
  void setup(void);
  const std::string &type(void) const { return type_; }
  void makeUserEquations(std::list<Assignment *>::iterator &p);
  void pre(Calculatable::SolutionMode solutionMode, int level);
  void post(Calculatable::SolutionMode solutionMode, int level);
  // non-mandatory
  // bool supportsSimultaneous(void) { return false; }
  // int maximumIterations(void) { return 1; }
  int sequential(void);
private:
  static std::string type_;
  std::string stagename(int i);  
  const std::string reteout(void);
  const std::string permout(void);
};  // class MembraneMulti22LinearCounterflow

#endif // MEMBRANEMULTILINEAR_H
